<?php

class Classement extends Model { // classe de modèle pour l'affichage de classements
	public $classement, $lejeu, $slug; // on définit les attributs de la classe
	
	public static function getFromSlug($slug) { // méthode permettant de récupérer le classement relatif à un jeu donné de la BDD
		$db = Database::getInstance(); // on récupère la connexion à la BDD
		$sql = "SELECT * FROM R_joueur_jeu WHERE jeu = :slug ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
		$req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
		$req->setFetchMode(PDO::FETCH_CLASS, "Classement"); // règle le type dans lequel on doit mettre les données (PDO va chercher les attributs de la classe de même nom que ses colonnes de BDD et mettre dedans les valeurs correspondantes)
		$req->execute(array(":slug" => $slug)); // on remplace le "::slug" par sa valeur dans la requête
		$res = array();
		while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
			$perso = Profil::getFromSlug($ligne['joueur']);
			$res[] = array(	'slug' => $ligne['joueur'],
							'pseudo' => $perso->pseudo,
							'avatar' => $perso->avatar ? BASE_URL."/images/avatar/".$perso->slug.".jpg": BASE_URL."/images/profil_42.jpg",
							'score' => $ligne['score'],
							'classement' => $ligne['classement']);
		}
		return $res;
    }
	
	public static function getGlobal() {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_jeu ORDER BY joueur";
        $req = $db->prepare($sql);
        $req->setFetchMode(PDO::FETCH_CLASS, "Classement");
        $req->execute();
        $res = array();
        $scores = array();
        $perso = array();
        $j = 0;
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) {
            $perso[$j] = Profil::getFromSlug($ligne['joueur']);
            $scoresj = array();
            while($ligne['joueur'] == $perso[$j]->slug){
                array_push($scoresj, $ligne['score']);
                $ligne = $req->fetch(PDO::FETCH_ASSOC);
            }
            $scores[$j] = array_sum($scoresj)/count($scoresj);
            $j++;
        }
        arsort($scores);
        $i = 1;
        foreach ($scores as $j => $score){
            array_push($res, array('slug' => $perso[$j]->slug,
							'pseudo' => $perso[$j]->pseudo,
							'avatar' => $perso[$j]->avatar ? BASE_URL."/images/avatar/".$perso[$j]->slug.".jpg": BASE_URL."/images/profil_42.jpg",
							'score' => $score,
							'classement' => $i));
            $i++;
        }
        return $res;
    }

	public static function getClassementJoueur($slug) {
		$db = Database::getInstance(); // on récupère la connexion à la BDD
		$sql = "SELECT * FROM R_joueur_jeu WHERE joueur LIKE :slug ORDER BY classement LIMIT 5";
		$req = $db->prepare($sql);
		$req->execute(array(":slug" => $slug));
		$res_top = array();
		while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
			$jeu = Jeu::getFromSlug($ligne['jeu']);
			$res_top[] = array(	'slug' => $ligne['jeu'],
							'nom' => $jeu->nom,
							'score' => $ligne['score'],
							'classement' => $ligne['classement']);
		}
		$sql = "SELECT * FROM R_joueur_jeu WHERE joueur = :slug ORDER BY classement DESC LIMIT 5";
		$req = $db->prepare($sql);
		$req->execute(array(":slug" => $slug));
		$res_down = array();
		while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
			$jeu = Jeu::getFromSlug($ligne['jeu']);
			$res_down[] = array(	'slug' => $ligne['jeu'],
							'nom' => $jeu->nom,
							'score' => $ligne['score'],
							'classement' => $ligne['classement']);
		}
		$sql = "SELECT * FROM R_joueur_jeu WHERE joueur LIKE :slug AND est_jeu_favoris";
		$req = $db->prepare($sql);
		$req->execute(array(":slug" => $slug));
		$res_fav = array();
		while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
			$jeu = Jeu::getFromSlug($ligne['jeu']);
			$res_fav[] = array(	'slug' => $ligne['jeu'],
								'nom' => $jeu->nom);
		}
		return array($res_top, $res_down, $res_fav);
	}
}
